#ifndef LIBPV_H
#define LIBPV_H

#include <stdio.h>
#include <inttypes.h>

#define PV_PATH_MAX (255)

typedef struct {
	char	_data[52];
} pv_engine_t;


typedef enum pv_bs_store_type {
	PV_STORE_STANDARD,
	PV_STORE_8BIT,
} pv_store_type_t;


typedef struct pv_engine_attr {
	char			desc_filename[PV_PATH_MAX + 1];
	char			store_filename[PV_PATH_MAX + 1];
	pv_store_type_t		store_type;
	uint16_t		jvisu_port;
} pv_engine_attr_t;


/* start/stop/stuff */
extern int	pv_init			(pv_engine_t		*engine,
					 pv_engine_attr_t	*attr);

extern int	pv_destroy		(pv_engine_t		*engine_anon);

extern int	pv_engine_attr_init	(pv_engine_attr_t	*attr);

extern int	pv_engine_attr_init_from_cmdline
					(pv_engine_attr_t	*attr,
					 int			argc,
					 char			*argv[]);

extern int	pv_debuglevel		(pv_engine_t		*engine,
					 int			level);


/* registration stuff */
extern uint8_t	*pv_register_uint8	(pv_engine_t		*engine,
					 const char		*varname);

extern int8_t	*pv_register_int8	(pv_engine_t		*engine,
					 const char		*varname);

extern uint16_t	*pv_register_uint16	(pv_engine_t		*engine,
					 const char		*varname);

extern int16_t	*pv_register_int16	(pv_engine_t		*engine,
					 const char		*varname);

extern uint32_t	*pv_register_uint32	(pv_engine_t		*engine,
					 const char		*varname);

extern int32_t	*pv_register_int32	(pv_engine_t		*engine,
					 const char		*varname);

extern float	*pv_register_float	(pv_engine_t		*engine,
					 const char		*varname);

extern double	*pv_register_double	(pv_engine_t		*engine,
					 const char		*varname);

extern char	*pv_register_string	(pv_engine_t		*engine,
					 const char		*varname);

extern uint8_t	*pv_register_uint8_array (pv_engine_t		*engine,
					 const char		*varname);

extern int8_t	*pv_register_int8_array	(pv_engine_t		*engine,
					 const char		*varname);

extern uint16_t	*pv_register_uint16_array
					(pv_engine_t		*engine,
					 const char		*varname);

extern int16_t	*pv_register_int16_array(pv_engine_t		*engine,
					 const char		*varname);

extern uint32_t	*pv_register_uint32_array
					(pv_engine_t		*engine,
					 const char		*varname);

extern int32_t	*pv_register_int32_array
					(pv_engine_t		*engine,
					 const char		*varname);

extern float	*pv_register_float_array
					(pv_engine_t		*engine,
					 const char		*varname);

extern double	*pv_register_double_array
					(pv_engine_t		*engine,
					 const char		*varname);


/* string handling stuff */
extern int	pv_cmp_string		(pv_engine_t		*engine,
					 const char		*string,
					 char			*s2);

extern int	pv_get_string		(pv_engine_t		*engine,
					 const char		*string,
					 char			*dest,
					 size_t			n);

extern int	pv_set_string		(pv_engine_t		*engine,
					 char			*string,
					 char			*src);

/* extern int	pv_set_nstring		(pv_engine_t		*engine, */
/* 					 char			*string, */
/* 					 char			*src, */
/* 					 size_t			n); */

/* set-a-default */
extern int	pv_default		(pv_engine_t		*engine,
					 void			*var);

/* transfer pv from and to device */
extern int	pv_to_device		(pv_engine_t		*engine,
					 void			*var);

extern int	pv_from_device		(pv_engine_t		*engine,
					 void			*var);

extern int	pv_set_fn_read		(pv_engine_t		*engine,
					 void			*var,
					 int			(*device_read)(pv_engine_t *engine, 
						 		               void *pv_void, 
									       unsigned long data));

extern int	pv_set_fn_write		(pv_engine_t		*engine,
					 void			*var,
					 int			(*device_write)(pv_engine_t *engine, 
						 		                void *pv_void, 
										unsigned long data));

extern int pv_device_read_int_file_ascii(pv_engine_t		*engine,
					 void 			*pv_void, 
					 unsigned long 		data);

extern int pv_device_write_int_file_ascii(pv_engine_t		*engine,
					  void 			*pv_void,
				  	  unsigned long 	data);

/* lock-stuff */
extern void	pv_lock			(pv_engine_t		*engine);
extern void	pv_unlock		(pv_engine_t		*engine);
extern void	pv_flush		(pv_engine_t		*engine,
					 void			*var,
					 size_t			n);

#endif	/* !LIBPV_H */
